import 'package:dartz/dartz.dart';
import 'package:data_db/data_db.dart';
import 'package:test/test.dart';

void main() {
  group('IntegerDataField', () {
    var field = IntegerDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue(80);
        expect(field.value, some(80));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue(800);
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some(800));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, 800);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), '800');
      });

      test('setValue', () {
        field.setValue(110);
        expect(field.value, some(110));
      });
    });
  });
  group('StringDataField', () {
    var field = StringDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue('80');
        expect(field.value, some('80'));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue('800');
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some('800'));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, '800');
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), '"800"');
      });

      test('setValue', () {
        field.setValue('110');
        expect(field.value, some('110'));
      });
    });
  });

  group('NumericDataField', () {
    var field = NumericDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue(80.896);
        expect(field.value, some(80.896));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue(800.6978);
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some(800.6978));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, 800.6978);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), '800.6978');
      });

      test('setValue', () {
        field.setValue(800.6978);
        expect(field.value, some(800.6978));
      });
    });
  });

  group('BooleanDataField', () {
    var field = BooleanDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue(false);
        expect(field.value, some(false));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue(true);
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some(true));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, true);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'true');
      });

      test('setValue', () {
        field.setValue(true);
        expect(field.value, some(true));
      });
    });
  });
  ////////////
  group('DateDataField', () {
    var field = DateDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue(DateTime(2019, 12, 1, 12));
        expect(field.value, some(DateTime(2019, 12, 1)));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue(DateTime(2019, 1, 1, 12));
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some(DateTime(2019, 1, 1)));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, DateTime(2019, 1, 1));
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), '"2019-01-01"');
      });

      test('setValue', () {
        field.setValue(DateTime(2020, 12, 1, 8));
        expect(field.value, some(DateTime(2020, 12, 1)));
      });
    });
  });

  group('DateTimeDataField', () {
    var field = DateTimeDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue(DateTime(2019, 12, 1, 12, 15, 8));
        expect(field.value, some(DateTime(2019, 12, 1, 12, 15, 8)));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue(DateTime(2019, 1, 1, 12, 15, 8));
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some(DateTime(2019, 1, 1, 12, 15, 8)));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, DateTime(2019, 1, 1, 12, 15, 8));
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), '"2019-01-01 12:15:08"');
      });

      test('setValue', () {
        field.setValue(DateTime(2020, 1, 1, 8, 58, 59));
        expect(field.value, some(DateTime(2020, 1, 1, 8, 58, 59)));
      });
    });
  });

  ////
  group('TimeDataField', () {
    var field = TimeDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue(DateTime(2019, 12, 1, 12, 15, 8));
        expect(field.value, some(DateTime(0, 0, 0, 12, 15, 8)));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue(DateTime(2019, 1, 1, 12, 15, 8));
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some(DateTime(0, 0, 0, 12, 15, 8)));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, DateTime(0, 0, 0, 12, 15, 8));
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), '"12:15:08"');
      });

      test('setValue', () {
        field.setValue(DateTime(2020, 1, 1, 8, 58, 59));
        expect(field.value, some(DateTime(0, 0, 0, 8, 58, 59)));
      });
    });
  });
  ////
  group('DynamicDataField', () {
    var field = DynamicDataField();
    group('Empty', () {
      test('isNotNull', () {
        expect(field.isNotNull, isFalse);
      });

      test('isNull', () {
        expect(field.isNull, isTrue);
      });

      test('value', () {
        expect(field.value, none());
      });

      test('tryGetValue', () {
        expect(field.tryGetValue((value) => true), isFalse);
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), 'null');
      });

      test('setValue', () {
        field.setValue(DateTime(2019, 12, 1, 12, 15, 8));
        expect(field.value, some(DateTime(2019, 12, 1, 12, 15, 8)));
      });
    });

    group('not Empty', () {
      setUp(() {
        field.setValue('Aloha!');
      });
      test('isNotNull', () {
        expect(field.isNotNull, isTrue);
      });

      test('isNull', () {
        expect(field.isNull, isFalse);
      });

      test('value', () {
        expect(field.value, some('Aloha!'));
      });

      test('tryGetValue', () {
        var x;
        field.tryGetValue((value) => x = value);
        expect(x, 'Aloha!');
      });

      test('toJsonValue', () {
        expect(field.toJsonValue(), '"Aloha!"');
      });

      test('setValue', () {
        field.setValue(80.258);
        expect(field.value, some(80.258));
      });
    });
  });
}
