import 'package:dartz/dartz.dart';
import 'package:data_db/data_db.dart';
import 'package:test/test.dart';

void main() {
  group('Dataset', () {
    group('Empty', () {
      var dataset = Dataset();

      test('structure', () {
        expect(dataset.structure, {});
      });

      test('count', () {
        expect(dataset.count, 0);
      });

      test('isEmpty', () {
        expect(dataset.isEmpty, isTrue);
      });

      test('isNotempty', () {
        expect(dataset.isNotempty, isFalse);
      });

      test('fields', () {
        expect(dataset.fields, {});
      });

      test('first', () {
        expect(dataset.first(), isFalse);
      });

      test('last', () {
        expect(dataset.last(), isFalse);
      });

      test('next', () {
        expect(dataset.next(), isFalse);
      });

      test('previous', () {
        expect(dataset.last(), isFalse);
      });

      test('fieldByName', () {
        expect(dataset.fieldByName('test'), none());
      });

      test('trySetValue', () {
        expect(dataset.trySetValue('test', 'test'), isFalse);
      });

      test('tryGetValue', () {
        expect(dataset.tryGetValue('test', (value) => true), isFalse);
      });

      test('toJson', () {
        expect(dataset.toJson(), '[]');
      });
    });

    group('With 1 field and 0 record', () {
      var dataset = Dataset();

      dataset.structure.addAll({'id': IntegerDataField});

      test('structure', () {
        expect(dataset.structure.isNotEmpty, isTrue);
      });

      test('count', () {
        expect(dataset.count, 0);
      });

      test('isEmpty', () {
        expect(dataset.isEmpty, isTrue);
      });

      test('isNotempty', () {
        expect(dataset.isNotempty, isFalse);
      });

      test('fields', () {
        expect(dataset.fields.isEmpty, isTrue);
      });

      test('first', () {
        expect(dataset.first(), false);
      });

      test('last', () {
        expect(dataset.last(), false);
      });

      test('next', () {
        expect(dataset.next(), false);
      });

      test('previous', () {
        expect(dataset.previous(), false);
      });

      test('fieldByName', () {
        expect(dataset.fieldByName('id').isSome(), false);
      });

      test('trySetValue error', () {
        expect(dataset.trySetValue('id', '1'), false);
      });

      test('trySetValue success', () {
        expect(dataset.trySetValue('id', 1), false);
      });

      test('tryGetValue', () {
        var x = 0;
        dataset.tryGetValue('id', (value) {
          x = value;
          return true;
        });
        expect(x, 0);
      });

      test('toJson', () {
        expect(dataset.toJson(), '[]');
      });
    });

    group('With 1 field and 1 record', () {
      var dataset = Dataset();

      dataset.structure.addAll({'id': IntegerDataField});
      dataset.newRecord();

      test('structure', () {
        expect(dataset.structure.isNotEmpty, isTrue);
      });

      test('count', () {
        expect(dataset.count, 1);
      });

      test('isEmpty', () {
        expect(dataset.isEmpty, isFalse);
      });

      test('isNotempty', () {
        expect(dataset.isNotempty, isTrue);
      });

      test('fields', () {
        expect(dataset.fields.isEmpty, false);
      });

      test('first', () {
        expect(dataset.first(), isTrue);
      });

      test('last', () {
        expect(dataset.last(), isTrue);
      });

      test('next', () {
        expect(dataset.next(), isFalse);
      });

      test('previous', () {
        expect(dataset.previous(), isFalse);
      });

      test('fieldByName', () {
        expect(dataset.fieldByName('id').isSome(), true);
      });

      test('trySetValue error', () {
        expect(dataset.trySetValue('id', '1'), isFalse);
      });

      test('trySetValue success', () {
        expect(dataset.trySetValue('id', 1), isTrue);
      });

      test('tryGetValue', () {
        var x = 0;
        dataset.tryGetValue('id', (value) {
          x = value;
          return true;
        });
        expect(x, 1);
      });

      test('forEach', () {
        var i = 0;
        dataset.forEach((record) {
          record.valueAsInt('id').map((a) => i += a);
        });
        expect(i, 1);
      });

      test('toJson', () {
        expect(dataset.toJson(), '[{"id": 1}]');
      });

      test('deleteRecord', () {
        dataset.deleteRecord();
        expect(dataset.count, 0);
      });
    });

    group('With 2 field and 2 record', () {
      var dataset = Dataset();

      dataset.structure.addAll({'id': IntegerDataField});
      dataset.structure.addAll({'description': StringDataField});
      dataset.newRecord();
      dataset.newRecord();

      test('structure', () {
        expect(dataset.structure.isNotEmpty, isTrue);
      });

      test('count', () {
        expect(dataset.count, 2);
      });

      test('isEmpty', () {
        expect(dataset.isEmpty, isFalse);
      });

      test('isNotempty', () {
        expect(dataset.isNotempty, isTrue);
      });

      test('fields', () {
        expect(dataset.fields.isEmpty, false);
      });

      test('first', () {
        expect(dataset.first(), isTrue);
      });

      test('last', () {
        expect(dataset.last(), isTrue);
      });

      test('next', () {
        expect(dataset.next(), isFalse);
      });

      test('previous', () {
        expect(dataset.previous(), isTrue);
      });

      test('fieldByName', () {
        expect(dataset.fieldByName('id').isSome(), true);
        expect(dataset.fieldByName('description').isSome(), true);
      });

      test('trySetValue error', () {
        expect(dataset.trySetValue('id', '1'), isFalse);
        expect(dataset.trySetValue('description', 1), isFalse);
      });

      test('trySetValue success', () {
        expect(dataset.trySetValue('id', 1), true);
        expect(dataset.trySetValue('description', '1'), true);
      });

      test('tryGetValue', () {
        var x = 0;
        dataset.tryGetValue('id', (value) {
          x = value;
          return true;
        });

        var y = '0';
        dataset.tryGetValue('description', (value) {
          y = value;
          return true;
        });
        expect(x, 1);
        expect(y, '1');
      });

      test('toJson', () {
        dataset.next();
        expect(dataset.trySetValue('id', 2), true);
        expect(dataset.trySetValue('description', '2'), true);
        expect(dataset.toJson(),
            '[{"id": 1, "description": "1"}, {"id": 2, "description": "2"}]');
      });

      test('forEach', () {
        var i = 0;
        var x = '';

        dataset.forEach((record) {
          record.valueAsInt('id').map((a) => i += a);
          record.valueAsString('description').map((a) => x += a);
        });
        expect(i, 3);
        expect(x, '12');
      });
      test('deleteRecord', () {
        dataset.deleteRecord();
        expect(dataset.count, 1);
      });

      test('clearFields', () {
        dataset.clearFields();
        expect(dataset.count, 0);
      });

      test('clearStructure', () {
        dataset.clearStructure();
        expect(dataset.structure.length, 0);
      });
    });
  });
}
